from . import baseml_bp
from flask import render_template, jsonify,request
from .config import *
import json,re

@baseml_bp.route('/test')
def test():
    return jsonify({'message': 'test success!'})

@baseml_bp.route('/get_epoch',methods=['GET'])
def get_epoch():
    return jsonify({'metrics': global_varibles['metrics'],'value': global_varibles['value']})

@baseml_bp.route('/select_task',methods=['POST'])
def select_task():
    if request.method == 'POST':
        task = request.json.get("task")
        print("task option",task)
        set_task(task=task)
        print("task now ",global_varibles["task"])
        response_data = {'message': '设置成功!', 'selected_option': task}
        return jsonify(response_data)

@baseml_bp.route('/select_model',methods=['POST'])
def select_model():
    if request.method == 'POST':
        model = request.json.get("model")
        print("model option",model)
        model = re.search(r'（(.*?)）', model).group(1)
        print('send:', model)
        set_model(model=model)
        print("model now ",global_varibles["model"])
        response_data = {'message': '设置成功!', 'selected_option': model}
        return jsonify(response_data)




@baseml_bp.route('/dataset',methods=['GET'])
def dataset():
    return render_template('dataset.html')

@baseml_bp.route('/select_dataset',methods=['POST'])
def select_dataset():
    data = json.loads(request.data)
    dataset = data.get("dataset")
    # 取dataset最后一个文件夹名字以及父文件夹名字
    # 如果路径包含/
    if '/' in  dataset:
        dir_name = dataset.split("/")[len(dataset.split("/"))-2]
        file_name = dataset.split("/")[-1]
    else:
        dir_name = dataset.split("\\")[len(dataset.split("\\"))-2]
        file_name = dataset.split("\\")[-1]
    print("dir_name:",dir_name)
    print("file_name:",file_name)
    dataset = os.path.join(dir_name,file_name)
    print("dataset:",dataset)
    set_dataset(dataset=dataset)
    set_dataset_path(dataset_path=dataset)
    update_dataset_path()
    print("dataset now ",global_varibles["dataset"])
    print("dataset_path now ",global_varibles["dataset_path"])
    # path = pip_settings['workfolder'] + "/checkpoints/basenn_model/"
    path = os.path.join(pip_settings['workfolder'],"checkpoints","baseml_model")
    if not os.path.exists(os.path.join(path,dir_name)):
            os.makedirs(os.path.join(path,dir_name))
    return jsonify({'message': '设置成功!', 'success': True})


@baseml_bp.route('/set_base_cfg',methods=['POST'])
def set_base_cfg():
    if request.method == 'POST':
        try:
            random_seed = request.form['random_seed']
            if update_global_varibles(random_seed=random_seed):
                response_data = {'message': '设置成功!', 'success': True}
            else:
                response_data = {'message': '设置失败!', 'success': False}
        except:
            pass
        try:
            param = request.form['param']
            if update_global_varibles(para=param):
                response_data = {'message': '设置成功!', 'success': True}
            else:
                response_data = {'message': '设置失败!', 'success': False}
        except:
            pass
        try:
            metrics = request.form['metrics']
            if update_global_varibles(metrics=metrics):
                response_data = {'message': '设置成功!', 'success': True}
            else:
                response_data = {'message': '设置失败!', 'success': False}
        except:
            pass
        return jsonify(response_data)
    



@baseml_bp.route('/get_dataset',methods=['GET'])
def get_baseml_dataset():
    print("get_dataset",get_all_dataset())
    return jsonify({'dataset': get_all_dataset()})





import PySimpleGUI as sg
import os
import threading

def convert_pth_2_onnx(pth_path,window):
    import torch
    output_text = f"将.pth模型文件转换为.onnx格式文件：{pth_path}\n"
    window["output"].update(output_text, append=True)
    # 输出路径就在pth文件所在目录下，用pth文件的原始文件名作为输出文件名
    output_path = f'{pth_path.split(".")[0]}.onnx'
    model = torch.load(pth_path, map_location='cpu')
    meta_info = model['meta'] if 'meta' in model else None
    if meta_info is None:
        print("Meta info not found!")
        window["output"].update("Meta info not found!\n", append=True)
        # return "Meta info not found! Please check your .pth file!"
    if meta_info['tool'] == 'BaseNN':
        print("BaseNN model detected!")
        window["output"].update("BaseNN model detected!\n", append=True)
        from BaseNN import nn
        nn_model = nn()
        nn_model.convert(checkpoint=pth_path, out_file=output_path)
        print(f"Convert successfully! Output path: {output_path}")
        window["output"].update(f"Convert successfully! Output path: {output_path}\n", append=True)
        # return f"Convert successfully! Output path: {output_path}"
    elif 'MMEdu' in meta_info['tool']:
        print("MMEdu model detected!")
        window["output"].update("MMEdu model detected!\n", append=True)
        backbone = meta_info['backbone']
        task = meta_info['task']
        if task=="Classification":
            print("Classification task detected!")
            window["output"].update("Classification task detected!\n", append=True)
            from MMEdu import MMClassification as cls
            cls_model = cls(backbone=backbone)
            cls_model.convert(checkpoint=pth_path, out_file=output_path)
            print(f"Convert successfully! Output path: {output_path}")
            window["output"].update(f"Convert successfully! Output path: {output_path}\n", append=True)
            # return f"Convert successfully! Output path: {output_path}"
        elif task=="Detection":
            print("Detection task detected!")
            window["output"].update("Detection task detected!\n", append=True)
            from MMEdu import MMDetection as det
            det_model = det(backbone=backbone)
            det_model.convert(checkpoint=pth_path, out_file=output_path)
            print(f"Convert successfully! Output path: {output_path}")
            window["output"].update(f"Convert successfully! Output path: {output_path}\n", append=True)
            # return f"Convert successfully! Output path: {output_path}"
        else:
            print("Task not supported yet!")
            window["output"].update("Task not supported yet!\n", append=True)
            # return "Task not supported yet!"
    else:
        print("Tool not supported yet!")
        window["output"].update("Tool not supported yet!\n", append=True)
        # return "Tool not supported yet!"

# 可用，但是放弃使用。
# def convert_all_folder(folder_path,window):
#     import os
#     for file in os.listdir(folder_path):
#         if file.endswith('.pth'):
#             pth_path = os.path.join(folder_path, file)
#             code = convert_pth_2_onnx(pth_path,window)
#             # if code == 1:
#             #     print(f'{file} convert successfully!')
#             # else:
#             #     print(f'{file} convert failed!')
#         else:
#             window["output"].update(f'{file} is not a .pth file!\n', append=True)
            # print(f'{file} is not a .pth file!')



# 定义处理函数2
def process_file_option2(file_path):
    # 在这里执行模型操作2，这里仅仅打印文件路径作为示例
    output_text = f"该功能目前不支持，敬请期待！\n"
    window["output"].update(output_text, append=True)

 
# 定义GUI布局
layout = [
    [sg.Text("选择要处理的文件：")],
    [sg.InputText(key="file_path"), sg.FileBrowse(button_text="选择单个模型文件")],
    # [sg.InputText(key="folder_path"), sg.FolderBrowse(button_text="选择模型文件夹")],
    [sg.Text("选择操作选项：")],
    [sg.Radio("pth转onnx", "options", default=True, key="option1"), sg.Radio("onnx转pth", "options", key="option2")],
    [sg.Button("模型转换",size=(20, 1))], # 按钮要居中显示
    [sg.Multiline(size=(60, 20), key="output", autoscroll=True)],
    [sg.Button("退出")]
]

# 创建窗口
window = sg.Window("Easy Convert", layout,font=("AnyFont", 13))
window.finalize()

window.set_min_size((200, 200))
# 事件循环
def main():
    while True:
        event, values = window.read()

        if event == sg.WINDOW_CLOSED or event == "退出":
            break
        elif event == "模型转换":
            file_path = values["file_path"]
            option1_selected = values["option1"]
            option2_selected = values["option2"]

            if os.path.exists(file_path) and file_path is not None:
                if option1_selected:
                    # use multithreading to avoid blocking the GUI
                    thread_id = threading.Thread(
                        target=convert_pth_2_onnx,
                        args=(file_path,window,),
                        daemon=True)
                    thread_id.start() 
                    # convert_pth_2_onnx(file_path)
                elif option2_selected:
                    process_file_option2(file_path)
            elif file_path is not None:
                window["output"].update("模型文件路径无效，请选择一个有效文件。\n", append=True)
            # if os.path.exists(folder_path):
            #     if option1_selected:
            #         # use multithreading to avoid blocking the GUI
            #         thread_id = threading.Thread(
            #             target=convert_all_folder,
            #             args=(folder_path,window,),
            #             daemon=True)
            #         thread_id.start()
            #         # convert_pth_2_onnx(file_path)
            #     elif option2_selected:
            #         # process_file_option2(file_path)
            #         pass
            # else:
            #     window["output"].update("模型文件夹路径无效，请选择一个有效文件夹。\n", append=True)

        # 关闭窗口
    window.close()





# -*- coding: UTF-8 -*-
from flask import Flask,render_template,jsonify,redirect,url_for

import json
from multiprocessing import Process,Pipe 
import time
import os
import subprocess
from .apis.mmedu.config import set_mmedu_checkpoints_path,generate_mmedu_code
from .apis.baseml.config import set_baseml_checkpoints_path,generate_baseml_code
from .apis.baseml.config import global_varibles as baseml_global_varibles
from .extensions import app,socketio
from .apis.basenn.config import set_basenn_checkpoints_path,generate_basenn_code,back2pwd,global_varibles

from .apis.mmedu.config import global_varibles as mmedu_global_varibles



@app.route('/')
def index():
    return redirect(url_for('mmedu.index'),code=301)
@app.route('/baseml/')
def baseml():
    return render_template('basemlPage.html',dataset=baseml_global_varibles['dataset'])
@app.route('/basenn/')
def basenn():
    return render_template('basennPage.html',dataset=global_varibles['dataset'])



mmedu_shared_data = {
    'message':None,
    'IsRunning':False,
    'time_stamp':'',
    'train_times':0,
    "pid":None
}
baseml_shared_data = {
    'message':None,
    'IsRunning':False,
    'time_stamp':'',
    'train_times':0,
    "pid":None
}
basenn_shared_data = {
    'message':None,
    'IsRunning':False,
    'time_stamp':'',
    'train_times':0,
    "pid":None
}


mmedu_running_process = None
basenn_running_process = None
baseml_running_process = None



def mmedu_train_task(child_conn,workfolder):
    global mmedu_shared_data
    mmedu_shared_data['IsRunning'] = True
    global mmedu_running_process
    print("training_thread")
    mmedu_shared_data['message'] = "正在训练 ……"
    mmedu_shared_data['IsRunning'] = True
    print("isRunning",mmedu_shared_data['IsRunning'])
    import sys
    mmedu_running_process = subprocess.Popen([f"{sys.executable}",os.path.join(workfolder,"mmedu_code.py")],stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    print(mmedu_running_process.pid)
    child_conn.send(mmedu_running_process.pid)
    out,error = mmedu_running_process.communicate() # 尝试打印运行时的报错
    mmedu_shared_data['IsRunning'] = False
    if error:
    # 将error编码为utf-8
        try:
            error = error.decode('utf-8')
        except:
            error = error.decode('gbk')
        print("error",error)
    print("subprocess end")
    
def baseml_train_task():
    global baseml_shared_data
    baseml_shared_data['IsRunning'] = True
    global baseml_running_process
    print("training_thread")
    baseml_shared_data['message'] = "正在训练 ……"
    import sys
    mmedu_running_process = subprocess.Popen([f"{sys.executable}",os.path.join(workfolder,"baseml_code.py")],stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    #baseml_poll_log_socket(baseml_running_process)
    print("subprocess end")
    baseml_shared_data['IsRunning'] = False
    print("isRunning",baseml_shared_data['IsRunning'])

def basenn_train_task():
    global basenn_shared_data
    basenn_shared_data['IsRunning'] = True
    global basenn_running_process
    print("training_thread")
    print(global_varibles)
    print(basenn_shared_data)
    import sys
    # basenn_running_process = subprocess.Popen([f"{sys.executable}",f"{workfolder}/basenn_code.py"],stdout=subprocess.PIPE, stderr=subprocess.PIPE,encoding='gb18030')
    basenn_running_process = subprocess.Popen([f"{sys.executable}",os.path.join(workfolder,"basenn_code.py")],stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    # 获取子进程输出
    basenn_poll_log_socket(basenn_running_process)
    basenn_running_process = None
    basenn_shared_data['IsRunning'] = False




@socketio.on('log')
def mmedu_poll_log_socket():
    global mmedu_shared_data
    time_stamp = mmedu_shared_data.get('time_stamp', '')
    last_line_num = 0
    print("log_task："+time_stamp)
    from .apis.mmedu.config import pip_settings as mmedu_pip_settings
    # log_path = mmedu_pip_settings['workfolder'] + "/my_checkpoints/" + "mmedu_"+time_stamp
    log_path = os.path.join(mmedu_pip_settings['workfolder'],"my_checkpoints","mmedu_"+time_stamp)
    while True:
        json_files = [x for x in os.listdir(log_path) if x.endswith('.json')]
        if len(json_files) != mmedu_shared_data['train_times']: # 防止多次训练时，没读取到最新的日志文件
            time.sleep(1)
        else:
            log_path = os.path.join(log_path, json_files[-1])
            break
    print("log_path",log_path)
    total_epoch = mmedu_global_varibles['epoch']
    while mmedu_shared_data['IsRunning']:
        if os.path.exists(log_path):
            with open(log_path, 'r') as f:
                lines = f.readlines()
                if len(lines) > last_line_num:
                    for line in lines[last_line_num:]:
                        log = json.loads(line)
                        log = json.dumps(log)
                        socketio.emit('log',log)
                        # print(log)
                        if last_line_num >1 and "val" in log:
                            log = json.loads(log)
                            # print("log",log)
                            if log['epoch'] == total_epoch:
                                print("log_task end")
                                mmedu_shared_data['IsRunning'] = False
                                break
                    last_line_num = len(lines)
                time.sleep(1)
        else:
            print("log_path not exist")
    print("log_task end")

@socketio.on('log')
def baseml_poll_log_socket(baseml_running_process):
    print("baseml_poll_log_socket")
    flag=True
    while flag:
        line = baseml_running_process.stdout.readline()
        error = baseml_running_process.stderr.read()
        if error:
            # 将error编码为utf-8
            try:
                error = error.decode('utf-8')
            except:
                error = error.decode('gbk')
            print("error",error)
            return jsonify({'message': error})
        if line:
            try:
                socketio.emit('log1',line.decode('utf-8'))
            except:
                socketio.emit('log1',line.decode('gbk'))
        else:
            flag=False
            print("log_task end")
            break
    return

@socketio.on('log')
def basenn_poll_log_socket(basenn_running_process):
    print("basenn_poll_log_socket")
    flag=True
    while flag:
        line = basenn_running_process.stdout.readline()
        error = basenn_running_process.stderr.read()
        if error:
            # 将error编码为utf-8
            try:
                error = error.decode('utf-8')
            except:
                error = error.decode('gbk')
            print("error",error)
            return jsonify({'message': error})
        if line:
            try:
                socketio.emit('log1',line.decode('utf-8'))
            except:
                socketio.emit('log1',line.decode('gbk'))
        else:
            flag=False
            print("log_task end")
            break
    return
running_process = None
@app.route('/baseml/start_thread',methods=['GET'])
def baseml_start_thread():
    global baseml_shared_data
    baseml_shared_data['train_times'] += 1
    global baseml_running_process
    if baseml_running_process and baseml_running_process.poll() is None:
        print("已经有一个模型在训练")
        return jsonify({'message': '已经有一个模型在训练'})
    else:
        baseml_shared_data['IsRunning'] = True
        if baseml_shared_data['IsRunning']:
            print("start_thread")
        baseml_train_task()
        return jsonify({'message': '训练已经开始'})


@app.route('/baseml/stop_thread',methods=['GET'])
def baseml_stop_thread():
    if baseml_running_process and baseml_running_process.poll() is None:
        baseml_running_process.terminate()
        return jsonify({'message': '已结束训练','success':True})
    else:
        return jsonify({'message': '没有正在训练的模型','success':False})

@app.route('/basenn/start_thread',methods=['GET'])
def basenn_start_thread():
    global basenn_shared_data
    basenn_shared_data['train_times'] += 1
    global basenn_running_process
    if basenn_running_process and basenn_running_process.poll() is None:
        print("已经有一个模型在训练")
        return jsonify({'message': '已经有一个模型在训练'})
    else:
        basenn_shared_data['IsRunning'] = True
        if basenn_shared_data['IsRunning']:
            print("start_thread")
        basenn_train_task()
        return jsonify({'message': '训练已经开始'})


@app.route('/basenn/stop_thread',methods=['GET'])
def basenn_stop_thread():
    if basenn_running_process and basenn_running_process.poll() is None:
        basenn_running_process.terminate()
        return jsonify({'message': '已结束训练','success':True})
    else:
        return jsonify({'message': '没有正在训练的模型','success':False})



running_process = None

@app.route('/mmedu/start_thread',methods=['GET'])
def mmedu_start_thread():
    global mmedu_shared_data
    mmedu_shared_data['train_times'] += 1
    global mmedu_running_process
    if mmedu_running_process and mmedu_running_process.poll() is None:
        print("已经有一个模型在训练")
        return jsonify({'message': '已经有一个模型在训练'})
    else:
        mmedu_shared_data['IsRunning'] = True
        parent_conn, child_conn = Pipe()
        running_process= Process(target=mmedu_train_task,args=(child_conn,workfolder,))
        running_process.start()
        # print("parent process get id",parent_conn.recv())
        train_pid = parent_conn.recv()
        mmedu_shared_data['pid'] = train_pid
        # mmedu_train_task()
        mmedu_poll_log_socket()
        return jsonify({'message': '训练已经开始'})


@app.route('/mmedu/stop_thread',methods=['GET'])
def mmedu_stop_thread():
    global mmedu_shared_data
    print(mmedu_shared_data)
    # 根据pid结束进程 
    if mmedu_shared_data['pid']:
        if os.name == 'nt':
            os.system("taskkill /pid "+str(mmedu_shared_data['pid'])+" /f")
        # 如果是linux系统，则使用下面的命令
        elif os.name == 'posix': 
            os.system("kill -9 "+str(mmedu_shared_data['pid']))
        mmedu_shared_data['pid'] = None
        return jsonify({'message': '已结束训练'},{'success':True})
    else:
        return jsonify({'message': '没有正在训练的模型','success':False})



@app.route('/mmedu/get_code',methods=['GET'])
def get_mmedu_code():
    global mmedu_shared_data
    print("get_mmedu_code")
    # make dir for checkpoints
    t = time.strftime('%Y%m%d_%H%M%S', time.localtime())
    from .apis.mmedu.config import pip_settings as mmedu_pip_settings
    # checkpoints_path = mmedu_pip_settings['workfolder'] + "/my_checkpoints/" + "mmedu_"+t
    checkpoints_path = os.path.join(mmedu_pip_settings['workfolder'],"my_checkpoints","mmedu_"+t)
    print("checkpoints_path: ",checkpoints_path)
    os.mkdir(checkpoints_path)
    set_mmedu_checkpoints_path(checkpoints_path=checkpoints_path)
    mmedu_shared_data['time_stamp'] = t
    print("time_stamp",mmedu_shared_data['time_stamp'])
    full_code = generate_mmedu_code()

    return jsonify(full_code)


@app.route('/baseml/get_code',methods=['GET'])
def get_baseml_code():
    print(1)
    global baseml_shared_data
    print("get_baseml_code")
    # make dir for checkpoints
    t = time.strftime('%Y%m%d_%H%M%S', time.localtime())
    from .apis.baseml.config import pip_settings as baseml_pip_settings
    # checkpoints_path = basenn_pip_settings['workfolder'] + "/my_checkpoints/"  + "basenn_"+t
    checkpoints_path = os.path.join(baseml_pip_settings['workfolder'],"my_checkpoints","baseml_"+t)
    print("checkpoints_path: ",checkpoints_path)
    os.mkdir(checkpoints_path)
    set_baseml_checkpoints_path(checkpoints_path=checkpoints_path)
    print("global_varibles",baseml_global_varibles)
    baseml_shared_data['time_stamp'] = t
    print("time_stamp",baseml_shared_data['time_stamp'])
    full_code = generate_baseml_code()
    return jsonify(full_code)


@app.route('/basenn/get_code',methods=['GET'])
def get_basenn_code():
    global basenn_shared_data
    print("get_basenn_code")
    # make dir for checkpoints
    t = time.strftime('%Y%m%d_%H%M%S', time.localtime())
    from .apis.basenn.config import pip_settings as basenn_pip_settings
    # checkpoints_path = basenn_pip_settings['workfolder'] + "/my_checkpoints/"  + "basenn_"+t
    checkpoints_path = os.path.join(basenn_pip_settings['workfolder'],"my_checkpoints","basenn_"+t)
    print("checkpoints_path: ",checkpoints_path)
    os.mkdir(checkpoints_path)
    set_basenn_checkpoints_path(checkpoints_path=checkpoints_path)
    print("global_varibles",global_varibles)
    basenn_shared_data['time_stamp'] = t
    print("time_stamp",basenn_shared_data['time_stamp'])
    full_code = generate_basenn_code()
    return jsonify(full_code)


@app.route('/get_xedu_pkg',methods=['GET'])
def get_xedu_pkg():
    import pkg_resources
    packages = [dist.project_name for dist in pkg_resources.working_set]
    res = {
        "MMEdu":False,
        "BaseML":False,
        "BaseNN":False,
    }
    # 如果packages中有BaseNN，则res中加入BaseNN:false,否则加入BaseNN:true
    if "BaseNN" in packages:
        res["BaseNN"] = True
    if "BaseML" in packages:
        res["BaseML"] = True
    if "MMEdu" in packages:
        res["MMEdu"] = True
    return jsonify(res)


def exist_or_mkdir(path):
    if not os.path.exists(path):
        os.mkdir(path)
    else:
        return

def check_workfolder(pwd):
    # 检查pwd下是否有datasets,checkpoints,my_checkpoints文件夹
    # exist_or_mkdir(pwd+"/datasets")
    exist_or_mkdir(os.path.join(pwd,"datasets"))
    # exist_or_mkdir(pwd+"/checkpoints")
    exist_or_mkdir(os.path.join(pwd,"checkpoints"))
    # exist_or_mkdir(pwd+"/my_checkpoints")
    exist_or_mkdir(os.path.join(pwd,"my_checkpoints"))
    # 检查datasets下是否有mmedu_cls,mmedu_det,basenn,basenn_文件夹
    # exist_or_mkdir(pwd+"/datasets/mmedu_cls")
    exist_or_mkdir(os.path.join(pwd,"datasets","mmedu_cls"))
    # exist_or_mkdir(pwd+"/datasets/mmedu_det")
    exist_or_mkdir(os.path.join(pwd,"datasets","mmedu_det"))
    # exist_or_mkdir(pwd+"/datasets/basenn")
    exist_or_mkdir(os.path.join(pwd,"datasets","basenn"))
    # 检查checkpoints下是否有mmedu_cls_model,mmedu_det_model,basenn_model文件夹
    # exist_or_mkdir(pwd+"/checkpoints/mmedu_cls_model")
    exist_or_mkdir(os.path.join(pwd,"checkpoints","mmedu_cls_model"))
    # exist_or_mkdir(pwd+"/checkpoints/mmedu_det_model")
    exist_or_mkdir(os.path.join(pwd,"checkpoints","mmedu_det_model"))
    # exist_or_mkdir(pwd+"/checkpoints/basenn_model")
    exist_or_mkdir(os.path.join(pwd,"checkpoints","basenn_model"))
    # 后续可做：下载预训练模型和数据集，放到对应文件夹下。
    


# 设置命令行参数解析
def get_args():
    import argparse
    parser = argparse.ArgumentParser(description='EasyTrain')
    parser.add_argument('--workfolder',default=os.getcwd(), help='workfolder path (default: current path)')
    return parser.parse_args()


workfolder = None
def main():
    pwd = get_args().workfolder
    check_workfolder(pwd)
    global workfolder
    workfolder = pwd
    from .apis.mmedu.config import pip_settings as mmedu_pip_settings
    from .apis.basenn.config import pip_settings as basenn_pip_settings
    from .apis.baseml.config import pip_settings as baseml_pip_settings
    mmedu_pip_settings["workfolder"] = pwd
    baseml_pip_settings["workfolder"] = pwd
    basenn_pip_settings["workfolder"] = pwd
    #basenn
    app.run(host='0.0.0.0',port=5000)


if __name__ == '__main__':
    # app.run(port=5000)
    # socketio.run(app,port=5000)
    # app.run(debug=True,port=5000)
    main()


from flask import Flask
from flask_socketio import SocketIO
from flask_cors import CORS
import os


from .apis.mmedu import mmedu_bp
from .apis.basenn import basenn_bp
from .apis.baseml import baseml_bp
app = Flask(__name__,static_url_path='/static')
app.config['JSON_AS_ASCII'] = False
CORS(app)

socketio = SocketIO(app,cors_allowed_origins="*")

app.register_blueprint(mmedu_bp)
app.register_blueprint(baseml_bp)
app.register_blueprint(basenn_bp)


def back2pwd(pwd,level):
    """
    返回上`level`数级目录的绝对路径
    """
    for i in range(level+1):
        pwd = os.path.abspath(os.path.dirname(pwd))
    return pwd


from . import mmedu_bp
from flask import render_template, request, jsonify
from .config import *
import json




@mmedu_bp.route('/index')
def index():
    return render_template('mmeduPage.html',
                        task = global_varibles["task"],
                        model = global_varibles["model"],
                        lr = global_varibles["lr"],
                        epoch = global_varibles["epoch"],
                        batch_size = global_varibles["batch_size"],
                        dataset = global_varibles["dataset"])



@mmedu_bp.route('/select_task',methods=['POST'])
def select_task():
    if request.method == 'POST':
        task = request.json.get("task")
        print("task option",task)
        set_task(task=task)
        print("task now ",global_varibles["task"])
        response_data = {'message': '设置成功!', 'selected_option': task}
        return jsonify(response_data)
    

@mmedu_bp.route('/select_model',methods=['POST'])
def select_model():
    if request.method == 'POST':
        model = request.json.get("model")
        print("model option",model)
        set_model(model=model)
        print("model now ",global_varibles["model"])
        response_data = {'message': '设置成功!', 'selected_option': model}
        return jsonify(response_data)


@mmedu_bp.route('/select_dataset',methods=['POST'])
def select_dataset():
    if request.method == 'POST':
        dataset = request.json.get("dataset")
        print("dataset option",dataset)
        set_dataset(dataset=dataset)
        print("dataset now ",global_varibles["dataset"])
        update_dataset_path()
        print("dataset_path now ",global_varibles["dataset_path"])
        response_data = {'message': '设置成功!', 'selected_option': dataset}
        task = global_varibles['task']
        if task=="classification":
            # 检查在checkpoints/mmcls_model/下是否存在该dataset同名文件夹,如果不存在，则创建该文件夹
            # path = pip_settings['workfolder'] + "/checkpoints/mmedu_cls_model/"
            path = os.path.join(pip_settings['workfolder'],"checkpoints","mmedu_cls_model")
            if not os.path.exists(os.path.join(path,dataset)):
                os.makedirs(os.path.join(path,dataset))
        elif task=="detection":
            # path = pip_settings['workfolder']  + "/checkpoints/mmedu_det_model/"
            path = os.path.join(pip_settings['workfolder'],"checkpoints","mmedu_det_model")
            if not os.path.exists(os.path.join(path,dataset)):
                os.makedirs(os.path.join(path,dataset))
        return jsonify(response_data)


@mmedu_bp.route('/set_base_cfg',methods=['POST'])
def set_base_config():
    # 从前端接收form表单数据
    lr = request.form['lr']
    batch_size = request.form['batch_size'] # 不设置batch_size，因为batch_size设置的数量不对，会使得训练报错，mmedu会自动设置batch_size
    epoch = request.form['epoch']
    random_seed = request.form['random_seed']
    set_lr(lr=lr)
    # set_batch_size(batch_size=batch_size)
    set_epoch(epoch=epoch)
    set_random_seed(random_seed=random_seed)
    print("set_config")
    print(global_varibles)
    response_data = {'message': '设置成功!'}
    return jsonify(response_data)


@mmedu_bp.route('/set_advance_cfg',methods=['POST'])
def set_advance_config():
    request_data = json.loads(request.data)
    optimizer = request_data['optimizer']
    weight_decay = request_data['weight_decay']
    class_num = request_data['class_num']
    device = request_data['device']
    pretrained_model = request_data['pretrained_model']
    if class_num != "":
        class_num = int(class_num)
        set_class_num(class_num=class_num)
    if weight_decay != "":
        weight_decay = float(weight_decay)
        set_weight_decay(weight_decay=weight_decay)
    set_optimizer(optimizer=optimizer)
    set_device(device=device)

    # 如果pretrained_model不是一个路径，而是一个预训练模型的名字，则需要更新pretrained_path
    # 判断pretrained_model是否是一个路径
    if pretrained_model !='':
        if pretrained_model.find('\\') == -1 and pretrained_model.find('/') == -1:
            # pretrained_model不是一个路径，而是一个预训练模型的名字，则需要更新pretrained_path
            update_pretrained_path(pretrained_model=pretrained_model)
        else:
            # pretrained_model是一个路径，则直接更新pretrained_path
            set_pretrained_path(pretrained_path=pretrained_model)
    return jsonify({'message': '设置成功!'})



@mmedu_bp.route('/get_local_dataset',methods=['GET'])
def get_local_dataset():
    if request.method == 'GET':
        print("getting_all_dataset")
        res = get_all_dataset()
        print(res)
        return jsonify(res)

@mmedu_bp.route('/get_local_pretrained_model',methods=['GET'])
def get_local_pretrained_model():
    if request.method == 'GET':
        print("getting_local_pretrained_model")
        pretrained_models = get_all_pretrained_model()
        if global_varibles['task'] == 'classification':
            res = pretrained_models['mmedu_cls_model']
            # 根据dataset_path的最后一个文件夹名字，获取对应的预训练模型
            dataset_path = global_varibles['dataset_path']
            # dataset_name = dataset_path.split('\\')[-1]
            dataset_name = os.path.basename(dataset_path)
            model_list = res[dataset_name]
            print(model_list)
            if len(model_list)==0:
                model_list.append('')
            return jsonify(model_list)
        elif global_varibles['task'] == 'detection':
            res = pretrained_models['mmedu_det_model']
            # 根据dataset_path的最后一个文件夹名字，获取对应的预训练模型
            dataset_path = global_varibles['dataset_path']
            # dataset_name = dataset_path.split('\\')[-1]
            dataset_name = os.path.basename(dataset_path)
            model_list = res[dataset_name]  # note: 安装包中的dataset名字与预训练模型中的文件夹名字需要保持一致
            print(model_list)
            if len(model_list)==0:
                model_list.append('')
            return jsonify(model_list)



@mmedu_bp.route('/get_epoch',methods=['GET'])
def get_epoch():
    return jsonify({'epoch': global_varibles['epoch']})

@mmedu_bp.route('/get_checkpoints_path',methods=['GET'])
def get_checkpoints_path():
    return jsonify({'checkpoints_path': global_varibles['checkpoints_path']})


@mmedu_bp.route('/convert_model',methods=['GET'])
def convert_model():
    save_path = global_varibles['checkpoints_path']
    # 遍历save_path下的所有pth文件，获取best，并转换为onnx
    # 1. 获取所有pth文件
    pth_list = []
    for root, dirs, files in os.walk(save_path):
        for file in files:
            if file.endswith('.pth'):
                pth_list.append(os.path.join(root, file))
    print(pth_list)
    # 2. 获取best, 以best_accuracy开头
    best_pth=""
    for pth in pth_list:
        # if pth.split('\\')[-1].startswith('best_accuracy'):
        #     best_pth = pth
        #     break
        if os.path.basename(pth).startswith('best_accuracy'):
            best_pth = pth
            break
    if global_varibles['task']=='classification':
        from MMEdu import MMClassification as cls
        cls_model = cls(backbone=global_varibles['model'])
        cls_model.convert(checkpoint=best_pth, out_file=f'{best_pth.split(".")[0]}.onnx')
        return jsonify({'message': '转换成功!', 'onnxpath': f'{best_pth.split(".")[0]}.onnx','success':True})
    elif global_varibles['task']=='detection':
        from MMEdu import MMDetection as det
        det_model = det(backbone=global_varibles['model'])
        det_model.convert(checkpoint=best_pth, out_file=f'{best_pth.split(".")[0]}.onnx')
        return jsonify({'message': '转换成功!', 'onnxpath': f'{best_pth.split(".")[0]}.onnx','success':True})    
    return jsonify({'message': '转换失败!', 'onnxpath': '','success':False})


import os
from flask import current_app


def back2pwd(pwd,level):
    """
    返回上`level`数级目录的绝对路径
    """
    for i in range(level+1):
        pwd = os.path.abspath(os.path.dirname(pwd))
    return pwd


# global_varibles = {
#     "dataset":"iris_training.csv",
#     "dataset_path": back2pwd(__file__,4) + "/datasets/basenn/iris/iris_training.csv",
#     "checkpoints_path": back2pwd(__file__,4) + "/my_checkpoints", # save fold path
#     "lr": 0.01,
#     "epochs": 10,
#     "network": [{'id': 1, 'type': 'linear', 'activation': 'relu', 'size': (4, 10)}, 
#                 {'id': 2, 'type': 'linear', 'activation': 'relu', 'size': (10, 20)},
#                   {'id': 3, 'type': 'linear', 'activation': 'softmax', 'size': (20, 3)}], # 网络结构，e.g.{"id":1, "name":'linear;,"size":(784,10),"activation":'relu'}
#     "pretrained_path": None,
#     "metrics": "acc", # options: acc mae mse
#     "loss":"CrossEntropyLoss", # options: CrossEntropyLoss MSELoss L1Loss……
#     "random_seed": 42,
#     "batch_size": 32,
#     "optimizer":"SGD"

# }




pip_settings = {
    "workfolder" : os.getcwd(), # pip包安装特有
}


global_varibles = {
    "dataset":os.path.join("iris","iris_training.csv"),
    # "dataset_path": pip_settings["workfolder"] + "/datasets/basenn/iris/iris_training.csv",
    "dataset_path":os.path.join(pip_settings["workfolder"],"datasets","basenn","iris","iris_training.csv"),
    "checkpoints_path": os.path.join(pip_settings["workfolder"],"my_checkpoints"), # save fold path
    "lr": 0.01,
    "epochs": 10,
    "network": [{'id': 1, 'type': 'linear', 'activation': 'relu', 'size': (4, 60)}, 
                {'id': 2, 'type': 'linear', 'activation': 'relu', 'size': (60, 30)},
                  {'id': 3, 'type': 'linear', 'activation': 'softmax', 'size': (30, 3)}], # 网络结构，e.g.{"id":1, "name":'linear;,"size":(784,10),"activation":'relu'}
    "pretrained_path": None,
    "metrics": "acc", # options: acc mae mse
    "loss":"CrossEntropyLoss", # options: CrossEntropyLoss MSELoss L1Loss……
    "random_seed": 42,
    "batch_size": 32,
    "optimizer":"Adam"
}

# global_varibles = {
#     "dataset":"test/train.csv",
#     "dataset_path": pip_settings["workfolder"]+ "/datasets/basenn/test/normed_train.csv",
#     "checkpoints_path": pip_settings["workfolder"]+ "/my_checkpoints", # save fold path
#     "lr": 0.01,
#     "epochs": 10,
#     "network": [{'id': 1, 'type': 'linear', 'activation': 'relu', 'size': (3, 60)}, 
#                 {'id': 2, 'type': 'linear', 'activation': 'relu', 'size': (60, 60)},
#                   {'id': 3, 'type': 'linear', 'activation': 'relu', 'size': (60, 6)},
#                   {'id': 4, 'type': 'linear', 'activation': 'None','size': (6, 1)},
#                   ],
#     "pretrained_path": None,
#     "metrics": "mse", # options: acc mae mse
#     "loss":"MSELoss", # options: CrossEntropyLoss MSELoss L1Loss……
#     "random_seed": 42,
#     "batch_size": 32,
#     "optimizer":"Adam"
# }




def get_all_pth(pwd):
    pth_list = []
    for file in os.listdir(pwd):
        if os.path.isdir(os.path.join(pwd,file)):
            pth_list.extend(get_all_pth(os.path.join(pwd,file)))
        else:
            if file.split(".")[-1] == "pth":
                pth_list.append(file)
    return pth_list


def get_all_pretrained_model():
    if os.name == "nt":
        pwd = os.path.join(pip_settings["workfolder"],"checkpoints","basenn_model",global_varibles['dataset'].split("\\")[0])
    else:
        pwd = os.path.join(pip_settings["workfolder"],"checkpoints","basenn_model",global_varibles['dataset'].split("/")[0])
    return get_all_pth(pwd)
    

def set_global_network(network):
    global_varibles["network"] = network

def set_batch_size(batch_size):
    global_varibles["batch_size"] = batch_size

def set_optimizer(optimizer):
    global_varibles["optimizer"] = optimizer

def set_dataset_path(dataset_path):
    global_varibles["dataset_path"] = dataset_path

def set_basenn_checkpoints_path(checkpoints_path):
    global_varibles["checkpoints_path"] = checkpoints_path

def set_lr(lr):
    global_varibles["lr"] = lr

def set_epochs(epochs):
    global_varibles["epochs"] = epochs

def set_pretrained_path(pretrained_path):
    global_varibles["pretrained_path"] = pretrained_path

def set_metrics(metrics):
    global_varibles["metrics"] = metrics

def set_loss(loss):
    global_varibles["loss"] = loss

def set_random_seed(random_seed):
    global_varibles["random_seed"] = random_seed

def set_dataset(dataset):
    global_varibles["dataset"] = dataset

def update_global_varibles(**kwargs):
    for k,v in kwargs.items():
        global_varibles[k] = v
    print("global_varibles now ",global_varibles)
    return True

def update_pretrained_path(pretrained):
    pwd = pip_settings["workfolder"]
    # pretrained_path = pwd + "/checkpoints/" + global_varibles['dataset'] + "/" + pretrained
    if os.name == "nt":
        pretrained_path = os.path.join(pwd,"checkpoints",global_varibles['dataset'].split("\\")[0],pretrained)
    else:
        pretrained_path = os.path.join(pwd,"checkpoints",global_varibles['dataset'].split("/")[0],pretrained)
    global_varibles['pretrained_path'] = pretrained_path

def update_dataset_path():
    # global_varibles["dataset_path"] = pip_settings["workfolder"] + "/datasets/" + global_varibles["dataset"]
    global_varibles["dataset_path"] = os.path.join(pip_settings["workfolder"],"datasets","basenn",global_varibles["dataset"])


def get_all_dataset():
    dataset_list = []
    # pwd = pip_settings["workfolder"] + "/datasets/basenn"
    pwd = os.path.join(pip_settings["workfolder"],"datasets","basenn")
    dirs = os.listdir(pwd)
    # print(dirs)
    for dir in dirs:
        for file in os.listdir(os.path.join(pwd,dir)):
            # print(os.path.join(pwd,dir,file))
            if os.path.isfile(os.path.join(pwd,dir,file)):
                dataset = os.path.join(pwd,dir,file)
                dataset_list.append(dataset)
    return dataset_list

def update_dataset_path():
    # global_varibles["dataset_path"] = pip_settings["workfolder"] + "/datasets/basenn/" + global_varibles["dataset"]
    global_varibles["dataset_path"] = os.path.join(pip_settings["workfolder"],"datasets","basenn",global_varibles["dataset"])



def _add_code(type,size,activation,**kwarg):
    if activation!="None":
        return f"model.add(layer='{type}',size={size},activation='{activation}')"
    else:
        return f"model.add(layer='{type}',size={size})"
    
def _add_optimizer(optimizer):
    return f"model.add(optimizer='{optimizer}')"


def check_if_regression():
    if global_varibles["metrics"] == "mse" or global_varibles["metrics"] == "mae":
        return True
    else:
        return False

def generate_basenn_code():
    update_dataset_path()
    full_code = ""
    # import
    import_part = "# coding:utf-8"+"\n"+"from BaseNN import nn" + "\n"
    def_part = "def generated_train():"+"\n"
    model_part = "\t"+"model = nn()" + "\n"
    dataset_part = ""
    # 如果dataset不是文件夹，那么设置dataset路径
    if check_if_regression():
        dataset_part = "\t"+f"model.load_tab_data(r'{global_varibles['dataset_path']}',y_type='float',batch_size={global_varibles['batch_size']})"+ "\n"
    else:
        dataset_part = "\t"+f"model.load_tab_data(r'{global_varibles['dataset_path']}',y_type='long',batch_size={global_varibles['batch_size']})"+ "\n"
    # save_fold
    save_part = "\t"+f"model.save_fold = r'{global_varibles['checkpoints_path']}'"+ "\n"
    # random_seed
    seed_part = "\t"+f"model.set_seed({global_varibles['random_seed']})" + "\n"
    # network
    construct_part=""
    train_part=""
    for n in global_varibles["network"]:
        construct_part += "\t"+_add_code(n["type"],n["size"],n["activation"]) + "\n"
    optimizer_part = "\t"+_add_optimizer(global_varibles["optimizer"]) + "\n"
    if global_varibles['pretrained_path'] is None:
        train_part = "\t"+f"model.train(epochs={global_varibles['epochs']},lr={global_varibles['lr']},loss='{global_varibles['loss']}',metrics=['{global_varibles['metrics']}'])" + "\n"
    else:
        train_part = "\t"+f"model.train(epochs={global_varibles['epochs']},lr={global_varibles['lr']},loss='{global_varibles['loss']}',metrics=['{global_varibles['metrics']}'],checkpoint='{global_varibles['pretrained_path']}')" + "\n"
    entry_part = "\n"+"if __name__ == '__main__':"+"\n"+"\t"+"generated_train()"+"\n"
    full_code = import_part + "\n" + def_part +model_part + dataset_part + save_part + seed_part + optimizer_part  + construct_part + train_part + entry_part
    with current_app.app_context():
        # with open(f"{pip_settings['workfolder']}/basenn_code.py","w") as f:
        with open(os.path.join(pip_settings['workfolder'],"basenn_code.py"),"w") as f:
            f.write(full_code)
        return full_code

# generate_code()


# coding:utf-8
from MMEdu import MMClassification as cls
def generated_train():
	model = cls(backbone='LeNet')
	model.num_classes = 3
	model.load_dataset(path=r'C:\Users\lizijian\Desktop\XEdu\datasets\mmedu_cls\hand_gray')
	model.save_fold = r'C:\Users\lizijian\Desktop\XEdu\my_checkpoints\mmedu_20231113_100004'
	model.train(epochs=10,validate=True,device='cpu',optimizer='SGD',lr=0.01, batch_size=None,weight_decay=0.001,checkpoint=None,random_seed=42)

if __name__ == '__main__':
	generated_train()



# coding:utf-8
from BaseNN import nn

def generated_train():
	model = nn()
	model.load_tab_data(r'C:\Users\lizijian\Desktop\XEdu\datasets\basenn\test\normed_train.csv',y_type='float',batch_size=32)
	model.save_fold = r'C:\Users\lizijian\Desktop\XEdu\my_checkpoints\basenn_20231113_153731'
	model.set_seed(42)
	model.add(optimizer='Adam')
	model.add(layer='linear',size=(3, 60),activation='relu')
	model.add(layer='linear',size=(60, 60),activation='relu')
	model.add(layer='linear',size=(60, 6),activation='relu')
	model.add(layer='linear',size=(6, 1))
	model.train(epochs=10,lr=0.01,loss='MSELoss',metrics=['mae'])

if __name__ == '__main__':
	generated_train()



import os
from flask import current_app

def back2pwd(pwd,level):
    """
    返回上`level`数级目录的绝对路径
    """
    for i in range(level+1):
        pwd = os.path.abspath(os.path.dirname(pwd))
    return pwd

pip_settings = {
    "workfolder" : os.getcwd(), # pip包安装特有
}

global_varibles = {
    "task":"reg",
    "model": "LinearRegression",
    "dataset":None,
    "dataset_path": back2pwd(__file__,3) + "?",
    "para":"",
    "checkpoints_path": back2pwd(__file__,3) + "/my_checkpoints", # save fold path
    "pretrained_path": None,
    "metrics": None, # options: acc mae mse
    "loss":"CrossEntropyLoss", # options: CrossEntropyLoss MSELoss L1Loss……
    "random_seed": "",
    "shuffle": True,
    "batch_size": 128,
    "optimizer":"Adam",
    "lr":0.01,
    "epochs":10,
    "value": 0,
}

# 设置每个参数，一个方法调整一个参数
def set_task(task):
    global_varibles["task"] = task

def set_model(model):
    global_varibles["model"] = model

def get_all_pkl(pwd):
    pth_list = []
    for file in os.listdir(pwd):
        if os.path.isdir(os.path.join(pwd,file)):
            pth_list.extend(get_all_pkl(os.path.join(pwd,file)))
        else:
            if file.split(".")[-1] == "pkl":
                pth_list.append(file)
    return pth_list

def set_batch_size(batch_size):
    global_varibles["batch_size"] = batch_size

def set_dataset_path(dataset_path):
    global_varibles["dataset_path"] = dataset_path

def set_baseml_checkpoints_path(checkpoints_path):
    global_varibles["checkpoints_path"] = checkpoints_path

def set_loss(loss):
    global_varibles["loss"] = loss

def set_random_seed(random_seed):
    global_varibles["random_seed"] = random_seed

def set_dataset(dataset):
    global_varibles["dataset"] = dataset

def update_global_varibles(**kwargs):
    for k,v in kwargs.items():
        global_varibles[k] = v
    print("global_varibles now ",global_varibles)
    return True

def update_dataset_path():
    # global_varibles["dataset_path"] = pip_settings["workfolder"] + "/datasets/" + global_varibles["dataset"]
    global_varibles["dataset_path"] = os.path.join(pip_settings["workfolder"],"datasets","basenn",global_varibles["dataset"])
    print(global_varibles['dataset'],global_varibles['dataset_path'])

def get_all_dataset():
    dataset_list = []
    # pwd = pip_settings["workfolder"] + "/datasets/basenn"
    pwd = os.path.join(pip_settings["workfolder"],"datasets","basenn")
    dirs = os.listdir(pwd)
    # print(dirs)
    for dir in dirs:
        for file in os.listdir(os.path.join(pwd,dir)):
            # print(os.path.join(pwd,dir,file))
            if os.path.isfile(os.path.join(pwd,dir,file)):
                dataset = os.path.join(pwd,dir,file)
                dataset_list.append(dataset)
    return dataset_list

def update_dataset_path():
    # global_varibles["dataset_path"] = pip_settings["workfolder"] + "/datasets/basenn/" + global_varibles["dataset"]
    global_varibles["dataset_path"] = os.path.join(pip_settings["workfolder"],"datasets","basenn",global_varibles["dataset"])

def _add_code(type,size,activation,**kwarg):
    if activation!="None":
        return f"model.add(layer='{type}',size={size},activation='{activation}')"
    else:
        return f"model.add(layer='{type}',size={size})"
    
def _add_optimizer(optimizer):
    return f"model.add(optimizer='{optimizer}')"

def check_if_regression():
    if global_varibles["metrics"] == "mse" or global_varibles["metrics"] == "mae":
        return True
    else:
        return False

def generate_baseml_code():
    update_dataset_path()
    full_code = ""
    # import
    if global_varibles['task'] == 'cls':
        import_part = "# coding:utf-8"+"\n"+"from BaseML import Classification as cls" + "\n"
    elif global_varibles['task'] == 'reg':
        import_part = "# coding:utf-8"+"\n"+"from BaseML import Regression as reg" + "\n"
    elif global_varibles['task'] == 'clt':
        import_part = "# coding:utf-8"+"\n"+"from BaseML import Cluster as clt" + "\n"
    else:
        import_part = "# coding:utf-8"+"\n"+"from BaseML import Regression as reg" + "\n"
    def_part = "def generated_train():"+"\n"
    
    if global_varibles['task'] == 'cls':
        model_part = "\t"+"model = cls('" + global_varibles['model'] + "')\n"
    elif global_varibles['task'] == 'reg':
        model_part = "\t"+"model = reg('" + global_varibles['model'] + "')\n"
    elif global_varibles['task'] == 'clt':
        model_part = "\t"+"model = clt('" + global_varibles['model'] + "')\n"
    else:
        model_part = "\t"+"model = reg('" + global_varibles['model'] + "')\n"
    # 如果dataset不是文件夹，那么设置dataset路径
    if len(global_varibles['random_seed'])>0:
        dataset_part = "\t"+f"model.load_tab_data(r'{global_varibles['dataset_path']}', random_seed={global_varibles['random_seed']})"+ "\n"
    else:
        dataset_part = "\t"+f"model.load_tab_data(r'{global_varibles['dataset_path']}')"+ "\n"
    # set_para
    if len(global_varibles['para'])>0:
        set_para_part = "\t"+f"model.set_para({global_varibles['para']})" + "\n"
    else:
        set_para_part = ""
    # train
    if global_varibles['model']=='MLP':
        train_part = "\t"+f"model.train(lr={global_varibles['lr']}, epochs={global_varibles['epochs']})" + "\n"+ f"\tmodel.save(r'{global_varibles['checkpoints_path']}/{global_varibles['task']}.pkl')\n"
    else:
        train_part = f"\tmodel.train()\n\tmodel.save(r'{global_varibles['checkpoints_path']}/{global_varibles['task']}.pkl')\n"

    valid_part = ''
    # valid
    if global_varibles['metrics']==None:
        if global_varibles['task'] == 'cls':
            global_varibles['metrics']='acc'
        elif global_varibles['task'] == 'reg':
            global_varibles['metrics']='r2'
        elif global_varibles['task'] == 'clt':
            global_varibles['metrics']='Silhouette Score'
    valid_part = f"\tmodel.valid(r'{global_varibles['dataset_path']}',metrics='{global_varibles['metrics']}')\n"

    entry_part = "\n"+"if __name__ == '__main__':"+"\n"+"\t"+"generated_train()"+"\n"
    full_code = import_part + "\n" + def_part +model_part + dataset_part + set_para_part + train_part + valid_part + entry_part
    with current_app.app_context():
        with open("baseml_code.py","w") as f:
            f.write(full_code)
        return full_code


# generate_code()


from . import basenn_bp
from flask import render_template, jsonify,request
from .config import *
import json


@basenn_bp.route('/test')
def test():
    return jsonify({'message': 'test success!'})


@basenn_bp.route('/dataset',methods=['GET'])
def dataset():
    return render_template('dataset.html')


@basenn_bp.route('/select_dataset',methods=['POST'])
def select_dataset():
    data = json.loads(request.data)
    dataset = data.get("dataset")
    # 取dataset最后一个文件夹名字以及父文件夹名字
    # 如果路径包含/
    if '/' in  dataset:
        dir_name = dataset.split("/")[len(dataset.split("/"))-2]
        file_name = dataset.split("/")[-1]
    else:
        dir_name = dataset.split("\\")[len(dataset.split("\\"))-2]
        file_name = dataset.split("\\")[-1]
    print("dir_name",dir_name)
    print("file_name",file_name)
    dataset = os.path.join(dir_name,file_name)
    print("dataset",dataset)
    set_dataset(dataset=dataset)
    update_dataset_path()
    print("dataset now ",global_varibles["dataset"])
    print("dataset_path now ",global_varibles["dataset_path"])
    # path = pip_settings['workfolder'] + "/checkpoints/basenn_model/"
    path = os.path.join(pip_settings['workfolder'],"checkpoints","basenn_model")
    if not os.path.exists(os.path.join(path,dir_name)):
            os.makedirs(os.path.join(path,dir_name))
    return jsonify({'message': '设置成功!', 'success': True})



@basenn_bp.route('/get_local_pretrained_model',methods=['GET'])
def get_local_pretrained_model():
    print("get_local_pretrained_model",get_all_pretrained_model())
    return jsonify({'pretrained_model': get_all_pretrained_model()})


@basenn_bp.route('/set_network',methods=['POST'])
def set_network():
    if request.method == 'POST':
        networks = request.json.get("network")
        print("network option",networks)
        print(type(networks))
        # 处理一下网络结构
        network = {}
        network_list = []
        for n in networks:
            print(n)
            network["id"] = n["id"]
            network["type"] = n["type"]
            network["activation"] = n["activation"]
            inputSize = int(n["inputSize"])
            outputSize = int(n["outputSize"])
            network["size"] = (inputSize,outputSize)
            network_list.append(network)
            network = {}
        set_global_network(network_list)
        print(network_list)
        print("network now ",global_varibles["network"])
        # set_network(network=network)
        # print("network now ",global_varibles["network"])
        response_data = {'message': '设置成功!', 'success': True}
        return jsonify(response_data)
    


@basenn_bp.route('/set_base_cfg',methods=['POST'])
def set_base_cfg():
    if request.method == 'POST':
        lr = request.form['lr']
        epochs = request.form['epochs']
        random_seed = request.form['random_seed']
        batch_size = request.form['batch_size']
        if update_global_varibles(lr=lr,epochs=epochs,random_seed=random_seed,batch_size=batch_size):
            response_data = {'message': '设置成功!', 'success': True}
        else:
            response_data = {'message': '设置失败!', 'success': False}
        return jsonify(response_data)
    

@basenn_bp.route('/set_advance_cfg',methods=['POST'])
def set_advance_cfg():
    if request.method == 'POST':
        request_data = json.loads(request.data)
        loss = request_data["loss"]
        metrics = request_data["metrics"]
        pretrained = request_data["pretrained"]
        set_loss(loss=loss)
        set_metrics(metrics=metrics)

        if pretrained != "None":
            update_pretrained_path(pretrained=pretrained)
        print("global_varibles now ",global_varibles)
        if pretrained.find('\\') == -1 and pretrained.find('/') == -1:
            # pretrained_model不是一个路径，而是一个预训练模型的名字，则需要更新pretrained_path
            if pretrained != "None":
                update_pretrained_path(pretrained=pretrained)
        else:
            # pretrained_model是一个路径，则直接更新pretrained_path
            set_pretrained_path(pretrained_path=pretrained)
    return jsonify({'message': '设置成功!', 'success': True})



@basenn_bp.route('/get_dataset',methods=['GET'])
def get_basenn_dataset():
    print("get_dataset",get_all_dataset())
    return jsonify({'dataset': get_all_dataset()})



@basenn_bp.route('/get_epoch',methods=['GET'])
def get_epoch():
    return jsonify({'epoch': global_varibles['epochs']})


@basenn_bp.route('/get_checkpoints_path',methods=['GET'])
def get_checkpoints_path():
    return jsonify({'checkpoints_path': global_varibles['checkpoints_path']})

@basenn_bp.route('/convert_model',methods=['GET'])
def convert_model():
    save_path = global_varibles['checkpoints_path']
    # 获取save_path下的.pth文件
    import os
    pth_files = [x for x in os.listdir(save_path) if x.endswith('.pth')]
    print("pth_files",pth_files)
    pth_path = os.path.join(save_path,pth_files[-1])
    print("pth_path",pth_path)
    from BaseNN import nn
    nn_model = nn()
    nn_model.convert(checkpoint=pth_path, out_file=f'{pth_path.split(".")[0]}.onnx')
    return jsonify({'message': '转换成功!', 'success': True,'onnxpath':f'{pth_path.split(".")[0]}.onnx'})



from setuptools import setup, find_packages

setup(
    name="easy-xedu",
    version="0.1.9",
    packages = find_packages(),
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        "flask",
        "flask-cors",
        "flask-socketio",
        "pysimplegui"
    ],
    python_requires='>=3.8',
    license="MIT",
    description="easy-xedu is a web-based platform for training deep learning models using MMEdu framework.",
    entry_points={'console_scripts': ['easytrain=EasyTrain.run:main','easyconvert=EasyConvert.easyconvert:main']},

    )


# config.py
import os
from flask import current_app

def back2pwd(pwd,level):
    """
    返回上`level`数级目录的绝对路径
    """
    for i in range(level+1):
        pwd = os.path.abspath(os.path.dirname(pwd))
    return pwd

pip_settings = {
    "workfolder" : os.getcwd(), # pip包安装特有
}


global_varibles = {
    "task": "classification",
    "model": "LeNet",
    "dataset": "hand_gray",
    "dataset_path": os.path.join(pip_settings["workfolder"],"datasets","mmedu_cls","hand_gray"),
    # "checkpoints_path": pip_settings["workfolder"] + "/my_checkpoints",
    "checkpoints_path": os.path.join(pip_settings["workfolder"],"my_checkpoints"),
    "lr": 0.01,
    "epoch": 10,
    "batch_size": None,
    "class_num":3,
    "validate" : True,
    "device": "cpu",
    "optimizer": "SGD",
    "weight_decay": 0.001,
    "random_seed": 42,
    "pretrained_path": None
}


model_list = {
    "classification":["LeNet","ResNet18","ResNet50","MobileNet"],
    "detection":["Yolov3","SSD_Lite","FasterRCNN"],

}

# 设置每个参数，一个方法调整一个参数
def set_task(task):
    global_varibles["task"] = task

def set_model(model):
    global_varibles["model"] = model

def set_lr(lr):
    global_varibles["lr"] = lr

def set_epoch(epoch):
    global_varibles["epoch"] = epoch

def set_batch_size(batch_size):
    global_varibles["batch_size"] = batch_size


def set_dataset_path(dataset_path):
    global_varibles["dataset_path"] = dataset_path

def set_mmedu_checkpoints_path(checkpoints_path):
    global_varibles["checkpoints_path"] = checkpoints_path

def set_dataset(dataset):
    global_varibles["dataset"] = dataset

def set_class_num(class_num):
    global_varibles["class_num"] = class_num

def set_device(device):
    global_varibles["device"] = device

def set_optimizer(optimizer):
    global_varibles["optimizer"] = optimizer

def weight_decay(weight_decay):
    global_varibles["weight_decay"] = weight_decay

def set_random_seed(random_seed):
    global_varibles["random_seed"] = random_seed

def set_weight_decay(weight_decay):
    global_varibles["weight_decay"] = weight_decay

def set_pretrained_path(pretrained_path):
    global_varibles["pretrained_path"] = pretrained_path


def get_all_dataset():
    res = {}
    # 获取cls文件夹下的所有文件夹
    # cls_dataset_path = pip_settings["workfolder"]  + "/datasets/mmedu_cls"
    cls_dataset_path = os.path.join(pip_settings["workfolder"],"datasets","mmedu_cls")
    cls_dataset_list = os.listdir(cls_dataset_path)
    # 过滤掉非文件夹
    # cls_dataset_list = [x for x in cls_dataset_list if os.path.isdir(cls_dataset_path + "/" + x)]
    cls_dataset_list = [x for x in cls_dataset_list if os.path.isdir(os.path.join(cls_dataset_path,x))]
    res['cls'] = [os.path.join(cls_dataset_path,x) for x in cls_dataset_list]
    # 获取det文件夹下的所有文件夹
    # det_dataset_path = pip_settings["workfolder"]  + "/datasets/mmedu_det"
    det_dataset_path = os.path.join(pip_settings["workfolder"],"datasets","mmedu_det")
    det_dataset_list = os.listdir(det_dataset_path)
    # 过滤掉非文件夹
    # det_dataset_list = [x for x in det_dataset_list if os.path.isdir(det_dataset_path + "/" + x)]  
    det_dataset_list = [x for x in det_dataset_list if os.path.isdir(os.path.join(det_dataset_path,x))]
    res['det'] = [os.path.join(det_dataset_path,x) for x in det_dataset_list]
    # print("dataset_list",res)
    return res


def get_all_pretrained_model():  # note： 预训练模型统一按照要求放在checkpoints文件夹的对应task文件夹的对应数据集文件夹下
    pwd = pip_settings["workfolder"] 
    # checkpoints文件夹
    # checkpoints_path = pwd + "/checkpoints"
    checkpoints_path = os.path.join(pwd,"checkpoints")
    print(checkpoints_path)
    checkpoints_list = os.listdir(checkpoints_path)
    # 过滤掉非文件夹
    # checkpoints_list = [x for x in checkpoints_list if os.path.isdir(checkpoints_path + "/" + x)]
    checkpoints_list = [x for x in checkpoints_list if os.path.isdir(os.path.join(checkpoints_path,x))]
    # print(checkpoints_list)
    res = {}
    for x in checkpoints_list:
        # 获取checkpoints文件夹下的所有文件夹
        # checkpoints_path = pwd + "/checkpoints/" + x
        checkpoints_path = os.path.join(pwd,'checkpoints',x)
        if os.path.isdir(checkpoints_path):
            checkpoints_list = os.listdir(checkpoints_path)
            temp = {}
            for y in checkpoints_list:
                # 获取checkpoints文件夹下的所有文件夹
                # checkpoints_path = pwd + "/checkpoints/" + x + "/" + y
                checkpoints_path = os.path.join(pwd,'checkpoints',x,y)
                if os.path.isdir(checkpoints_path):
                    checkpoints_list = os.listdir(checkpoints_path)
                    # 仅保留.pth文件
                    checkpoints_list = [x for x in checkpoints_list if x.endswith('.pth')]
                    temp[y] = checkpoints_list
                else:
                    continue
            res[x] = temp
        else:
            continue
    print(res)
    return res


def update_pretrained_path(pretrained_model):
    pwd = pip_settings["workfolder"]
    if pretrained_model == "None":
        global_varibles['pretrained_path'] = None
        return
    if global_varibles['task'] == 'classification':
        # pretrained_path = pwd + "/checkpoints/mmedu_cls_model/" + global_varibles['dataset'] + "/" + pretrained_model
        pretrained_path = os.path.join(pwd,"checkpoints","mmedu_cls_model",global_varibles['dataset'],pretrained_model)
        global_varibles['pretrained_path'] = pretrained_path
    elif global_varibles['task'] == 'detection':
        # pretrained_path = pwd + "/checkpoints/mmedu_det_model/" + global_varibles['dataset'] + "/" + pretrained_model
        pretrained_path = os.path.join(pwd,"checkpoints","mmedu_det_model",global_varibles['dataset'],pretrained_model)
        global_varibles['pretrained_path'] = pretrained_path

def update_dataset_path():
    if global_varibles['task'] == 'classification':
        # global_varibles["dataset_path"] = pip_settings["workfolder"]+ "/datasets/mmedu_cls/" + global_varibles["dataset"]
        global_varibles["dataset_path"] = os.path.join(pip_settings["workfolder"],"datasets","mmedu_cls",global_varibles["dataset"])
    elif global_varibles['task'] == 'detection':
        # global_varibles["dataset_path"] = pip_settings["workfolder"]+ "/datasets/mmedu_det/" + global_varibles["dataset"]
        global_varibles["dataset_path"] = os.path.join(pip_settings["workfolder"],"datasets","mmedu_det",global_varibles["dataset"])

def generate_mmedu_code():
    update_dataset_path()
    full_code=""
    if global_varibles['task'] == 'detection':
        import_part = "# coding:utf-8"+"\n"+"from MMEdu import MMDetection as det" + "\n"
        # write_part = "with open(\"pid.pkl\",\"w\") as f:"+"\n"+"\t"+"f.write(str(os.getpid()))"+"\n"
        def_part =  "def generated_train():"+"\n"
        construct_part = "\t"+f"model = det(backbone='{global_varibles['model']}')"+ "\n"
        class_part = "\t"+f"model.num_classes = {global_varibles['class_num']}"+ "\n"
        dataset_part = "\t"+f"model.load_dataset(path=r'{global_varibles['dataset_path']}')"+ "\n"
        save_part = "\t"+f"model.save_fold = r'{global_varibles['checkpoints_path']}'"+ "\n"
        if global_varibles['pretrained_path'] == None:
            train_part = "\t"+f"model.train(epochs={global_varibles['epoch']},validate={global_varibles['validate']},device='{global_varibles['device']}',optimizer='{global_varibles['optimizer']}',lr={global_varibles['lr']}, batch_size={global_varibles['batch_size']},weight_decay={global_varibles['weight_decay']},checkpoint={global_varibles['pretrained_path']},random_seed={global_varibles['random_seed']})" + "\n"+"\n"
        else:
            train_part = "\t"+f"model.train(epochs={global_varibles['epoch']},validate={global_varibles['validate']},device='{global_varibles['device']}',optimizer='{global_varibles['optimizer']}',lr={global_varibles['lr']}, batch_size={global_varibles['batch_size']},weight_decay={global_varibles['weight_decay']},checkpoint=r'{global_varibles['pretrained_path']}',random_seed={global_varibles['random_seed']})" + "\n"+"\n"
        entry_part = "if __name__ == '__main__':"+"\n"+"\t"+"generated_train()"+"\n"
        full_code = import_part + def_part + construct_part + class_part + dataset_part + save_part + train_part + entry_part
        # 写入另一个py文件
        with current_app.app_context():
            with open(f"{pip_settings['workfolder']}/mmedu_code.py","w") as f:
                f.write(full_code)

    elif global_varibles['task'] == 'classification':
        import_part = "# coding:utf-8"+"\n"+"from MMEdu import MMClassification as cls" + "\n"
        def_part =  "def generated_train():"+"\n"
        construct_part = "\t"+f"model = cls(backbone='{global_varibles['model']}')"+ "\n"
        class_part = "\t"+f"model.num_classes = {global_varibles['class_num']}"+ "\n"
        dataset_part = "\t"+f"model.load_dataset(path=r'{global_varibles['dataset_path']}')"+ "\n"
        save_part = "\t"+f"model.save_fold = r'{global_varibles['checkpoints_path']}'"+ "\n"
        if global_varibles['pretrained_path'] == None:
            train_part = "\t"+f"model.train(epochs={global_varibles['epoch']},validate={global_varibles['validate']},device='{global_varibles['device']}',optimizer='{global_varibles['optimizer']}',lr={global_varibles['lr']}, batch_size={global_varibles['batch_size']},weight_decay={global_varibles['weight_decay']},checkpoint={global_varibles['pretrained_path']},random_seed={global_varibles['random_seed']})" + "\n"+"\n"
        else:
            train_part = "\t"+f"model.train(epochs={global_varibles['epoch']},validate={global_varibles['validate']},device='{global_varibles['device']}',optimizer='{global_varibles['optimizer']}',lr={global_varibles['lr']}, batch_size={global_varibles['batch_size']},weight_decay={global_varibles['weight_decay']},checkpoint=r'{global_varibles['pretrained_path']}',random_seed={global_varibles['random_seed']})" + "\n"+"\n"
        entry_part = "if __name__ == '__main__':"+"\n"+"\t"+"generated_train()"+"\n"
        full_code = import_part + def_part + construct_part + class_part + dataset_part + save_part + train_part + entry_part
        with current_app.app_context():
            # with open(f"{pip_settings['workfolder']}/mmedu_code.py","w") as f:
            with open(os.path.join(pip_settings['workfolder'],"mmedu_code.py"),"w") as f:
                f.write(full_code)
    print("生成代码：",full_code)
    return full_code

# generate_code()